@@ -21,6 +21,7 @@ import glob
 import io
 import multiprocessing
 import os
+import pathlib
 import runpy
 import shutil
 import sys
@@ -31,9 +32,11 @@ from distutils.dir_util import copy_tree, remove_tree
 from pathlib import Path
 from unittest import TestCase, mock
 
+import pytest
+
 import pyfakefs.tests.import_as_example
+import pyfakefs.tests.logsio
 from pyfakefs import fake_filesystem_unittest, fake_filesystem
-from pyfakefs.extra_packages import pathlib
 from pyfakefs.fake_filesystem import OSType
 from pyfakefs.fake_filesystem_unittest import (
     Patcher, Pause, patchfs, PatchMode
@@ -96,8 +99,8 @@ class TestPyfakefsUnittest(TestPyfakefsUnittestBase):  # pylint: disable=R0904
         self.assertTrue(self.fs.exists('/fake_file.txt'))
         with open('/fake_file.txt') as f:
             content = f.read()
-        self.assertEqual(content, 'This test file was created using the '
-                                  'open() function.\n')
+        self.assertEqual('This test file was created using the '
+                         'open() function.\n', content)
 
     def test_io_open(self):
         """Fake io module is bound"""
@@ -108,8 +111,8 @@ class TestPyfakefsUnittest(TestPyfakefsUnittestBase):  # pylint: disable=R0904
         self.assertTrue(self.fs.exists('/fake_file.txt'))
         with open('/fake_file.txt') as f:
             content = f.read()
-        self.assertEqual(content, 'This test file was created using the '
-                                  'io.open() function.\n')
+        self.assertEqual('This test file was created using the '
+                         'io.open() function.\n', content)
 
     def test_os(self):
         """Fake os module is bound"""
@@ -120,22 +123,21 @@ class TestPyfakefsUnittest(TestPyfakefsUnittestBase):  # pylint: disable=R0904
     def test_glob(self):
         """Fake glob module is bound"""
         is_windows = sys.platform.startswith('win')
-        self.assertEqual(glob.glob('/test/dir1/dir*'),
-                         [])
+        self.assertEqual([], glob.glob('/test/dir1/dir*'))
         self.fs.create_dir('/test/dir1/dir2a')
         matching_paths = glob.glob('/test/dir1/dir*')
         if is_windows:
-            self.assertEqual(matching_paths, [r'\test\dir1\dir2a'])
+            self.assertEqual([r'/test/dir1\dir2a'], matching_paths)
         else:
-            self.assertEqual(matching_paths, ['/test/dir1/dir2a'])
+            self.assertEqual(['/test/dir1/dir2a'], matching_paths)
         self.fs.create_dir('/test/dir1/dir2b')
         matching_paths = sorted(glob.glob('/test/dir1/dir*'))
         if is_windows:
-            self.assertEqual(matching_paths,
-                             [r'\test\dir1\dir2a', r'\test\dir1\dir2b'])
+            self.assertEqual([r'/test/dir1\dir2a', r'/test/dir1\dir2b'],
+                             matching_paths)
         else:
-            self.assertEqual(matching_paths,
-                             ['/test/dir1/dir2a', '/test/dir1/dir2b'])
+            self.assertEqual(['/test/dir1/dir2a', '/test/dir1/dir2b'],
+                             matching_paths)
 
     def test_shutil(self):
         """Fake shutil module is bound"""
@@ -343,6 +345,13 @@ class NoSkipNamesTest(fake_filesystem_unittest.TestCase):
         with self.assertRaises(OSError):
             pyfakefs.tests.import_as_example.open_this_file()
 
+    def test_open_patched_in_module_ending_with_io(self):
+        # regression test for #569
+        file_path = '/foo/bar'
+        self.fs.create_file(file_path, contents=b'abc')
+        contents = pyfakefs.tests.logsio.file_contents(file_path)
+        self.assertEqual(b'abc', contents)
+
 
 class AdditionalSkipNamesTest(fake_filesystem_unittest.TestCase):
     """Make sure that modules in additional_skip_names are not patched.
@@ -532,7 +541,15 @@ class NoRootUserTest(fake_filesystem_unittest.TestCase):
             open(file_path, 'w')
 
 
-class PauseResumeTest(TestPyfakefsUnittestBase):
+class PauseResumeTest(fake_filesystem_unittest.TestCase):
+    def setUp(self):
+        self.real_temp_file = None
+        self.setUpPyfakefs()
+
+    def tearDown(self):
+        if self.real_temp_file is not None:
+            self.real_temp_file.close()
+
     def test_pause_resume(self):
         fake_temp_file = tempfile.NamedTemporaryFile()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
@@ -540,11 +557,11 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         self.pause()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
         self.assertFalse(os.path.exists(fake_temp_file.name))
-        real_temp_file = tempfile.NamedTemporaryFile()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.real_temp_file = tempfile.NamedTemporaryFile()
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         self.resume()
-        self.assertFalse(os.path.exists(real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_fs(self):
@@ -557,15 +574,15 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         self.fs.pause()
         self.assertTrue(self.fs.exists(fake_temp_file.name))
         self.assertFalse(os.path.exists(fake_temp_file.name))
-        real_temp_file = tempfile.NamedTemporaryFile()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.real_temp_file = tempfile.NamedTemporaryFile()
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         # pause does nothing if already paused
         self.fs.pause()
-        self.assertFalse(self.fs.exists(real_temp_file.name))
-        self.assertTrue(os.path.exists(real_temp_file.name))
+        self.assertFalse(self.fs.exists(self.real_temp_file.name))
+        self.assertTrue(os.path.exists(self.real_temp_file.name))
         self.fs.resume()
-        self.assertFalse(os.path.exists(real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_contextmanager(self):
@@ -575,10 +592,10 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         with Pause(self):
             self.assertTrue(self.fs.exists(fake_temp_file.name))
             self.assertFalse(os.path.exists(fake_temp_file.name))
-            real_temp_file = tempfile.NamedTemporaryFile()
-            self.assertFalse(self.fs.exists(real_temp_file.name))
-            self.assertTrue(os.path.exists(real_temp_file.name))
-        self.assertFalse(os.path.exists(real_temp_file.name))
+            self.real_temp_file = tempfile.NamedTemporaryFile()
+            self.assertFalse(self.fs.exists(self.real_temp_file.name))
+            self.assertTrue(os.path.exists(self.real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_fs_contextmanager(self):
@@ -588,10 +605,10 @@ class PauseResumeTest(TestPyfakefsUnittestBase):
         with Pause(self.fs):
             self.assertTrue(self.fs.exists(fake_temp_file.name))
             self.assertFalse(os.path.exists(fake_temp_file.name))
-            real_temp_file = tempfile.NamedTemporaryFile()
-            self.assertFalse(self.fs.exists(real_temp_file.name))
-            self.assertTrue(os.path.exists(real_temp_file.name))
-        self.assertFalse(os.path.exists(real_temp_file.name))
+            self.real_temp_file = tempfile.NamedTemporaryFile()
+            self.assertFalse(self.fs.exists(self.real_temp_file.name))
+            self.assertTrue(os.path.exists(self.real_temp_file.name))
+        self.assertFalse(os.path.exists(self.real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
     def test_pause_resume_without_patcher(self):
@@ -615,6 +632,7 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
             p.resume()
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
+        real_temp_file.close()
 
     def test_pause_resume_contextmanager(self):
         with Patcher() as p:
@@ -629,6 +647,7 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
                 self.assertTrue(os.path.exists(real_temp_file.name))
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
+        real_temp_file.close()
 
 
 class TestPyfakefsTestCase(unittest.TestCase):
@@ -716,9 +735,11 @@ class PathlibTest(TestCase):
     @patchfs
     def test_cwd(self, fs):
         """Make sure fake file system is used for os in pathlib"""
-        self.assertEqual(os.path.sep, str(pathlib.Path.cwd()))
+        is_windows = sys.platform.startswith('win')
+        root_dir = 'C:' + os.path.sep if is_windows else os.path.sep
+        self.assertEqual(root_dir, str(pathlib.Path.cwd()))
         dot_abs = pathlib.Path(".").absolute()
-        self.assertEqual(os.path.sep, str(dot_abs))
+        self.assertEqual(root_dir, str(dot_abs))
         self.assertTrue(dot_abs.exists())
 
 
@@ -779,13 +800,13 @@ class TestOtherFS(fake_filesystem_unittest.TestCase):
     def test_real_file_with_home(self):
         """Regression test for #558"""
         self.fs.is_windows_fs = os.name != 'nt'
+        if self.fs.is_windows_fs:
+            self.fs.is_macos = False
+        self.fs.reset()
         self.fs.add_real_file(__file__)
         with open(__file__) as f:
             self.assertTrue(f.read())
         home = Path.home()
-        if sys.version_info < (3, 6):
-            # fspath support since Python 3.6
-            home = str(home)
         os.chdir(home)
         with open(__file__) as f:
             self.assertTrue(f.read())
@@ -800,6 +821,12 @@ class TestOtherFS(fake_filesystem_unittest.TestCase):
         self.assertTrue(os.path.exists(path.upper()))
         self.assertTrue(os.path.ismount(r'\\share\foo'))
         self.assertTrue(os.path.ismount(r'C:'))
+        self.assertEqual('\\', os.sep)
+        self.assertEqual('\\', os.path.sep)
+        self.assertEqual('/', os.altsep)
+        self.assertEqual(';', os.pathsep)
+        self.assertEqual('\r\n', os.linesep)
+        self.assertEqual('nul', os.devnull)
 
     def test_linux(self):
         self.fs.os = OSType.LINUX
@@ -811,6 +838,12 @@ class TestOtherFS(fake_filesystem_unittest.TestCase):
         self.assertFalse(os.path.exists(path.upper()))
         self.assertTrue(os.path.ismount('/'))
         self.assertFalse(os.path.ismount('//share/foo'))
+        self.assertEqual('/', os.sep)
+        self.assertEqual('/', os.path.sep)
+        self.assertEqual(None, os.altsep)
+        self.assertEqual(':', os.pathsep)
+        self.assertEqual('\n', os.linesep)
+        self.assertEqual('/dev/null', os.devnull)
 
     def test_macos(self):
         self.fs.os = OSType.MACOS
@@ -822,6 +855,12 @@ class TestOtherFS(fake_filesystem_unittest.TestCase):
         self.assertTrue(os.path.exists(path.upper()))
         self.assertTrue(os.path.ismount('/'))
         self.assertFalse(os.path.ismount('//share/foo'))
+        self.assertEqual('/', os.sep)
+        self.assertEqual('/', os.path.sep)
+        self.assertEqual(None, os.altsep)
+        self.assertEqual(':', os.pathsep)
+        self.assertEqual('\n', os.linesep)
+        self.assertEqual('/dev/null', os.devnull)
 
     def test_drivelike_path(self):
         self.fs.os = OSType.LINUX
@@ -834,5 +873,23 @@ class TestOtherFS(fake_filesystem_unittest.TestCase):
         self.assertTrue(os.path.exists(str(file_path.relative_to(folder))))
 
 
+@unittest.skipIf(sys.platform != 'win32', 'Windows-specific behavior')
+class TestAbsolutePathOnWindows(fake_filesystem_unittest.TestCase):
+    @patchfs
+    def test_is_absolute(self, fs):
+        # regression test for #673
+        self.assertTrue(pathlib.Path(".").absolute().is_absolute())
+
+
+class TestModuleScopedFsWithTmpdir:
+    @pytest.fixture(autouse=True)
+    def test_internal(self, tmpdir):
+        yield
+
+    def test_fail(self, fs_module):
+        # Regression test for #684
+        assert True
+
+
 if __name__ == "__main__":
     unittest.main()
