@@ -93,8 +93,13 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
         # by the locale preferred encoding - which under Windows is
         # usually not UTF-8, but something like Latin1, depending on the locale
         text_fractions = 'Ümläüts'
-        with self.open(file_path, 'w') as f:
-            f.write(text_fractions)
+        try:
+            with self.open(file_path, 'w') as f:
+                f.write(text_fractions)
+        except UnicodeEncodeError:
+            # see https://github.com/jmcgeheeiv/pyfakefs/issues/623
+            self.skipTest("This test does not work with an ASCII locale")
+
         with self.open(file_path) as f:
             contents = f.read()
         self.assertEqual(contents, text_fractions)
@@ -113,8 +118,12 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
     def test_write_str_read_bytes(self):
         file_path = self.make_path('foo')
         str_contents = 'Äsgül'
-        with self.open(file_path, 'w') as f:
-            f.write(str_contents)
+        try:
+            with self.open(file_path, 'w') as f:
+                f.write(str_contents)
+        except UnicodeEncodeError:
+            # see https://github.com/jmcgeheeiv/pyfakefs/issues/623
+            self.skipTest("This test does not work with an ASCII locale")
         with self.open(file_path, 'rb') as f:
             contents = f.read()
         self.assertEqual(str_contents, contents.decode(
@@ -908,7 +917,7 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
             self.assertEqual(b'test', f.read())
 
     def test_unicode_filename(self):
-        file_path = self.make_path(u'тест')
+        file_path = self.make_path('тест')
         with self.open(file_path, 'wb') as f:
             f.write(b'test')
         with self.open(file_path, 'rb') as f:
@@ -921,6 +930,19 @@ class FakeFileOpenTest(FakeFileOpenTestBase):
             with self.open(self.os.devnull) as f:
                 self.assertEqual('', f.read())
 
+    def test_utf16_text(self):
+        # regression test for #574
+        file_path = self.make_path('foo')
+        with self.open(file_path, "w", encoding='utf-16') as f:
+            assert f.write("1") == 1
+
+        with self.open(file_path, "a", encoding='utf-16') as f:
+            assert f.write("2") == 1
+
+        with self.open(file_path, "r", encoding='utf-16') as f:
+            text = f.read()
+            assert text == "12"
+
 
 class RealFileOpenTest(FakeFileOpenTest):
     def use_real_fs(self):
@@ -1464,6 +1486,7 @@ class FakeFileOpenLineEndingTest(FakeFileOpenTestBase):
         with self.open(file_path, mode='r', newline='\r\n') as f:
             self.assertEqual(['1\r\n', '2\n3\r4'], f.readlines())
 
+    @unittest.skipIf(sys.version_info >= (3, 10), "U flag no longer supported")
     def test_read_with_ignored_universal_newlines_flag(self):
         file_path = self.make_path('some_file')
         file_contents = b'1\r\n2\n3\r4'
@@ -1475,6 +1498,14 @@ class FakeFileOpenLineEndingTest(FakeFileOpenTestBase):
         with self.open(file_path, mode='U', newline='\r') as f:
             self.assertEqual('1\r\n2\n3\r4', f.read())
 
+    @unittest.skipIf(sys.version_info < (3, 11), "U flag still supported")
+    def test_universal_newlines_flag_not_supported(self):
+        file_path = self.make_path('some_file')
+        file_contents = b'1\r\n2\n3\r4'
+        self.create_file(file_path, contents=file_contents)
+        with self.assertRaises(ValueError):
+            self.open(file_path, mode='U', newline='\r')
+
     def test_write_with_newline_arg(self):
         file_path = self.make_path('some_file')
         with self.open(file_path, 'w', newline='') as f:
@@ -1895,7 +1926,7 @@ class ResolvePathTest(FakeFileOpenTestBase):
             # unclear behavior with PyPi
             self.skip_real_fs()
         self.assert_raises_os_error(
-            [errno.ENOTDIR, errno.EBADF], self.os.chdir, 10)
+            [errno.ENOTDIR, errno.EBADF], self.os.chdir, 500)
         dir_path = self.make_path('foo', 'bar')
         self.create_dir(dir_path)
 
